#ifndef __TUM3D_CUDACOMPRESS__COMPRESS_HEIGHTFIELD_H__
#define __TUM3D_CUDACOMPRESS__COMPRESS_HEIGHTFIELD_H__


#include "global.h"

#include <vector>


namespace cudaCompress {

size_t compressHeightfieldGetRequiredMemory(const void* pCudaCompressInstance);
bool compressHeightfieldInit(void* pCudaCompressInstance);
bool compressHeightfieldShutdown(void* pCudaCompressInstance);

// Compress one level of a heightfield image:
// - perform integer (reversible) DWT
// - truncate and store least significant bit of lowpass coefficients
// - encode highpass coefficients into bitstream
// - return lowpass coefficients (minus last bit)
// The input is assumed to be roughly zero-centered, and shouldn't use more than ~10 bits.
// Decompress works analogously.
// NOTE: Decompress modifies the lowpass image! (by appending the bit that was removed during compression)
CUCOMP_DLL bool compressHeightfieldOneLevel(void* pCudaCompressInstance, const short* dpImage, uint sizeX, uint sizeY, short* dpLowpass, short lowpassShift, std::vector<uint>& highpassBitStream);
CUCOMP_DLL void decompressHeightfieldOneLevel(void* pCudaCompressInstance, short* dpImage, uint sizeX, uint sizeY, short* dpLowpass, short lowpassShift, const std::vector<uint>& highpassBitStream);
CUCOMP_DLL void decompressHeightfieldOneLevel(void* pCudaCompressInstance, short* dpImage, uint sizeX, uint sizeY, short* dpLowpass, short lowpassShift, const uint* pHighpassBits, uint highpassBitCount);

CUCOMP_DLL void getLowpassHeightfield(void* pCudaCompressInstance, const short* dpImage, uint sizeX, uint sizeY, short* dpLowpass, short lowpassShift);
CUCOMP_DLL void getHeightfieldMinMax(void* pCudaCompressInstance, const short* dpImage, uint sizeX, uint sizeY, short* dpValMin, short* dpValMax);
CUCOMP_DLL void shiftHeightfield(void* pCudaCompressInstance, short* dpImage, uint sizeX, uint sizeY, short shift);

}


#endif
