#ifndef __TUM3D_CUDACOMPRESS__COMPRESS_IMAGE_H__
#define __TUM3D_CUDACOMPRESS__COMPRESS_IMAGE_H__


#include "global.h"

#include <vector>


namespace cudaCompress {

size_t compressImageGetRequiredMemory(const void* pCudaCompressInstance);
bool compressImageInit(void* pCudaCompressInstance);
bool compressImageShutdown(void* pCudaCompressInstance);

// Compress/decompress one level of an image:
// - perform DWT
// - quantize and encode difference between lowpass coefficients and reference lowpass (i.e. reconstructed image from next level)
// - quantize and encode highpass coefficients into bitstream
// - compress performs roundtrip, i.e. dpImage is updated to the reconstructed image
// Processes the first channelCountToProcess channels, out of channelCountTotal channels
// (useful e.g. to process only RGB out of an RGBA image).
// Restrictions: 1 <= channelCountTotal <= 4 and channelCountToProcess <= channelCountTotal.
// Does not perform color space conversion!
CUCOMP_DLL bool compressImageOneLevel(void* pCudaCompressInstance, byte* dpImage, uint sizeX, uint sizeY, uint channelCountTotal, uint channelCountToProcess, const byte* dpLowpassReference, std::vector<uint>& highpassBitStream, float quantizationStep);
CUCOMP_DLL void decompressImageOneLevel(void* pCudaCompressInstance, byte* dpImage, uint sizeX, uint sizeY, uint channelCountTotal, uint channelCountToProcess, const byte* dpLowpassReference, const std::vector<uint>& highpassBitStream, float quantizationStep);
CUCOMP_DLL void decompressImageOneLevel(void* pCudaCompressInstance, byte* dpImage, uint sizeX, uint sizeY, uint channelCountTotal, uint channelCountToProcess, const byte* dpLowpassReference, const uint* pHighpassBits, uint highpassBitCount, float quantizationStep);

CUCOMP_DLL void getLowpassImage(void* pCudaCompressInstance, const byte* dpImage, uint sizeX, uint sizeY, uint channelCountTotal, uint channelCountToProcess, byte* dpLowpass);

}


#endif
