#ifndef __TUM3D_CUDACOMPRESS__COMPRESS_VOLUME_H__
#define __TUM3D_CUDACOMPRESS__COMPRESS_VOLUME_H__


#include "global.h"

#include <vector>


namespace cudaCompress {

size_t compressVolumeGetRequiredMemory(const void* pCudaCompressInstance);
bool compressVolumeInit(void* pCudaCompressInstance);
bool compressVolumeShutdown(void* pCudaCompressInstance);

// Compress one level of a scalar volume (lossless):
// - perform integer (reversible) DWT
// - encode highpass coefficients into bitstream
// - return lowpass coefficients
// The input is assumed to be roughly zero-centered, and shouldn't use more than ~10 bits.
// Decompress works analogously.
CUCOMP_DLL bool compressVolumeLosslessOneLevel(void* pCudaCompressInstance, const short* dpImage, uint sizeX, uint sizeY, uint sizeZ, short* dpLowpass, std::vector<uint>& highpassBitStream);
CUCOMP_DLL void decompressVolumeLosslessOneLevel(void* pCudaCompressInstance, short* dpImage, uint sizeX, uint sizeY, uint sizeZ, const short* dpLowpass, const std::vector<uint>& highpassBitStream);

// Convenience functions for multi-level lossless compression
CUCOMP_DLL bool compressVolumeLossless(void* pCudaCompressInstance, const short* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, std::vector<uint>& bitStream);
CUCOMP_DLL void decompressVolumeLossless(void* pCudaCompressInstance, short* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, const std::vector<uint>& bitStream);


// Compress a scalar volume (lossy):
// - perform numLevels DWT
// - encode coefficients into bitstream
// The input is assumed to be roughly zero-centered, and shouldn't use more than ~10 bits after quantization.
// Decompress works analogously.
CUCOMP_DLL bool compressVolume(void* pCudaCompressInstance, const byte* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, std::vector<uint>& bitStream, float quantizationStepLevel0);
CUCOMP_DLL void decompressVolume(void* pCudaCompressInstance, byte* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, const std::vector<uint>& bitStream, float quantizationStepLevel0);
CUCOMP_DLL void decompressVolume(void* pCudaCompressInstance, byte* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, const uint* pBits, uint bitCount, float quantizationStepLevel0);

CUCOMP_DLL bool compressVolumeFloat(void* pCudaCompressInstance, const float* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, std::vector<uint>& bitStream, float quantizationStepLevel0);
CUCOMP_DLL void decompressVolumeFloat(void* pCudaCompressInstance, float* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, const std::vector<uint>& bitStream, float quantizationStepLevel0);
CUCOMP_DLL void decompressVolumeFloat(void* pCudaCompressInstance, float* dpImage, uint sizeX, uint sizeY, uint sizeZ, uint numLevels, const uint* pBits, uint bitCount, float quantizationStepLevel0);

CUCOMP_DLL void getVolumeFloatMaxAbs(void* pCudaCompressInstance, const float* dpImage, uint sizeX, uint sizeY, uint sizeZ, float* dpValMax);

}


#endif
