#ifndef __TUM3D_CUDACOMPRESS__ENCODE_CPU_H__
#define __TUM3D_CUDACOMPRESS__ENCODE_CPU_H__


#include "global.h"

#include <string>
#include <vector>

#include "tools/BitStream.h"

#include "EncodeCommon.h"


namespace cudaCompress {

enum ECoder {
	CODER_RUNLENGTH_HUFFMAN = 0,
	CODER_HUFFMAN,
	CODER_RUNLENGTH_GOLOMBRICE,
	CODER_GOLOMBRICE,
	CODER_RBUC,
	CODER_COUNT
};
std::string getCoderName(ECoder coder);


bool encodeCPU(ECoder coder, BitStream* pBitStream, /*const*/ std::vector<Symbol>* pSymbolStream, uint offsetInterval);
bool decodeCPU(ECoder coder, BitStreamReadOnly* pBitStream, std::vector<Symbol>* pSymbolStream, uint symbolCount, uint offsetInterval);

bool encodeCPU(ECoder coder, BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount, uint offsetInterval);
bool decodeCPU(ECoder coder, BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount, uint offsetInterval);


bool encodeRLHuffCPU(BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount, uint offsetInterval, bool zeroRunsOnly = true);
bool decodeRLHuffCPU(BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount, uint offsetInterval, bool zeroRunsOnly = true);

bool encodeHuffCPU(BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount, uint offsetInterval);
bool decodeHuffCPU(BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount, uint offsetInterval);

bool encodeRLRiceCPU(BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount, uint offsetInterval, bool zeroRunsOnly = true);
bool decodeRLRiceCPU(BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount, uint offsetInterval, bool zeroRunsOnly = true);

bool encodeRiceCPU(BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount, uint offsetInterval);
bool decodeRiceCPU(BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount, uint offsetInterval);

bool encodeRBUCCPU(BitStream* ppBitStreams[], /*const*/ std::vector<Symbol>* ppSymbolStreams[], uint blockCount);
bool decodeRBUCCPU(BitStreamReadOnly* ppBitStreams[], std::vector<Symbol>* ppSymbolStreams[], uint symbolCountPerBlock, uint blockCount);

}


#endif
