#ifndef __TUM3D_CUDACOMPRESS__HUFFMAN_TABLE_H__
#define __TUM3D_CUDACOMPRESS__HUFFMAN_TABLE_H__


#include "global.h"

#include <vector>

#include "tools/BitStream.h"

#include "EncodeCommon.h"


namespace cudaCompress {

class Instance;


class HuffmanDecodeTable
{
public:
	HuffmanDecodeTable();
	HuffmanDecodeTable(HuffmanDecodeTable&& other);
	~HuffmanDecodeTable();

	HuffmanDecodeTable& operator=(HuffmanDecodeTable&& other);

	void clear();
	void readFromBitstream(BitStreamReadOnly& bitstream);

	static uint computeMaxGPUSize();
	uint computeGPUSize() const;
	void uploadToGPU(byte* dpTable) const;
	void uploadToGPUAsync(byte* dpTable) const;

	// indexed by codeword index
	Symbol* m_pSymbolTable;
	uint m_symbolTableSize;

	// indexed by codeword length
	// these are just pointers into m_pCodewordIndex
	int* m_pCodewordFirstIndexPerLength;
	int* m_pCodewordMinPerLength;
	int* m_pCodewordMaxPerLength;

private:
	static const int CODEWORD_INDEX_SIZE = 3 * MAX_CODEWORD_BITS;
	int* m_pCodewordIndex;

	void build(const std::vector<uint>& codewordCountPerLength);

	// don't allow copy or assignment
	HuffmanDecodeTable(const HuffmanDecodeTable&);
	void operator=(const HuffmanDecodeTable&);
};

class HuffmanEncodeTable
{
public:
	static size_t getRequiredMemory(const Instance* pInstance);
	static void init(Instance* pInstance);
	static void shutdown(Instance* pInstance);

	HuffmanEncodeTable();
	HuffmanEncodeTable(HuffmanEncodeTable&& other);
	~HuffmanEncodeTable();

	HuffmanEncodeTable& operator=(HuffmanEncodeTable&& other);

	void clear();
	static bool design(Instance* pInstance, HuffmanEncodeTable* pTables, uint tableCount, const Symbol** pdpSymbolStreams, uint* pSymbolCountPerStream);

	void uploadToGPU(uint* dpCodewords, uint* dpCodewordLengths) const;
	void uploadToGPUAsync(uint* dpCodewords, uint* dpCodewordLengths) const;

	void writeToBitstream(BitStream& bitstream) const;

	// data to be written to bitstream
	Symbol              m_symbolMax;
	std::vector<Symbol> m_symbols;
	std::vector<uint>   m_codewordCountPerLength;

private:
	void build(const uint* pSymbolProbabilities, uint distinctSymbolCount);

	// the actual encode table, indexed by symbol
	uint* m_pCodewords;
	uint* m_pCodewordLengths;
	uint m_codewordTableSize;

	// don't allow copy or assignment
	HuffmanEncodeTable(const HuffmanEncodeTable&);
	HuffmanEncodeTable& operator=(const HuffmanEncodeTable&);
};

}


#endif
