#ifndef __TUM3D_CUDACOMPRESS__HUFFMAN_TABLE_CPU_H__
#define __TUM3D_CUDACOMPRESS__HUFFMAN_TABLE_CPU_H__


#include "global.h"

#include <vector>

#include "tools/BitStream.h"

#include "EncodeCommon.h"


namespace cudaCompress {

class HuffmanTableCPU
{
public:
	void clear();
	void design(const std::vector<Symbol>& symbolStream);

	void writeToBitstream(BitStream& bitstream) const;
	void readFromBitstream(BitStreamReadOnly& bitstream);

	bool operator==(const HuffmanTableCPU& rhs) const { return m_symbols == rhs.m_symbols && m_codewordCountPerLength == rhs.m_codewordCountPerLength; }
	bool operator!=(const HuffmanTableCPU& rhs) const { return !(*this == rhs); }

	std::vector<Symbol> m_symbols;
	std::vector<uint>   m_codewordCountPerLength;
};

struct HuffmanEntry
{
	Symbol m_symbol;
	int    m_codeword;
	uint   m_codewordLength;
};

class HuffmanDecodeTableCPU
{
public:
	void clear();
	void build(const HuffmanTableCPU& table);

	// indexed by codeword index
	std::vector<HuffmanEntry> m_decodeTable;

	// indexed by codeword length
	std::vector<int> m_codewordFirstIndexPerLength;
	std::vector<int> m_codewordMinPerLength;
	std::vector<int> m_codewordMaxPerLength;
};

class HuffmanEncodeTableCPU
{
public:
	void clear();
	void build(const HuffmanDecodeTableCPU& decodeTable);

	// indexed by symbol
	std::vector<HuffmanEntry> m_encodeTable;
};

}


#endif
