#include "Quantize.h"

#include "cudaUtil.h"

#include "QuantizeKernels.cui"


namespace cudaCompress {


void quantize2D(byte* dpQuant, uint channelCount, uint channel, const float* dpData, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	quantize2Dkernel<<<blockCount, blockSize>>>(dpQuant + channel, channelCount, dpData, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("quantize2Dkernel execution failed");
}

void unquantize2D(float* dpData, const byte* dpQuant, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	unquantize2Dkernel<<<blockCount, blockSize>>>(dpData, dpQuant + channel, channelCount, sizeX, sizeY, rowPitchDst);
	cudaCheckMsg("unquantize2Dkernel execution failed");
}



void symbolize2D(ushort* dpSymbols, const short* dpData, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	symbolize2Dkernel<<<blockCount, blockSize>>>(dpSymbols, dpData, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("symbolize2Dkernel execution failed");
}

void unsymbolize2D(short* dpData, const ushort* dpSymbols, uint sizeX, uint sizeY, uint rowPitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	unsymbolize2Dkernel<<<blockCount, blockSize>>>(dpData, dpSymbols, sizeX, sizeY, rowPitchDst);
	cudaCheckMsg("unsymbolize2Dkernel execution failed");
}

void symbolize3D(ushort* dpSymbols, const short* dpData, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchSrc, uint slicePitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	uint blockCountZ = (sizeZ + blockSize.z - 1) / blockSize.z;
	dim3 blockCount(blockCountX, blockCountY, blockCountZ);

	symbolize3Dkernel<<<blockCount, blockSize>>>(dpSymbols, dpData, sizeX, sizeY, sizeZ, rowPitchSrc, slicePitchSrc);
	cudaCheckMsg("symbolize3Dkernel execution failed");
}

void unsymbolize3D(short* dpData, const ushort* dpSymbols, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchDst, uint slicePitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	uint blockCountZ = (sizeZ + blockSize.z - 1) / blockSize.z;
	dim3 blockCount(blockCountX, blockCountY, blockCountZ);

	unsymbolize3Dkernel<<<blockCount, blockSize>>>(dpData, dpSymbols, sizeX, sizeY, sizeZ, rowPitchDst, slicePitchDst);
	cudaCheckMsg("unsymbolize3Dkernel execution failed");
}


void quantizeToSymbols2D(ushort* dpQuant, const float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	quantizeToSymbols2Dkernel<<<blockCount, blockSize>>>(dpQuant, dpData, 1.0f / quantizationStep, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("quantizeToSymbols2Dkernel execution failed");
}

void quantizeToSymbolsRoundtrip2D(ushort* dpQuant, float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	quantizeToSymbolsRoundtrip2Dkernel<<<blockCount, blockSize>>>(dpQuant, dpData, quantizationStep, 1.0f / quantizationStep, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("quantizeToSymbolsRoundtrip2Dkernel execution failed");
}

void unquantizeFromSymbols2D(float* dpData, const ushort* dpQuant, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	unquantizeFromSymbols2Dkernel<<<blockCount, blockSize>>>(dpData, dpQuant, quantizationStep, sizeX, sizeY, rowPitchDst);
	cudaCheckMsg("unquantizeFromSymbols2Dkernel execution failed");
}

void quantizeToSymbols3D(ushort* dpQuant, const float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchSrc, uint slicePitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY * sizeZ);

	quantizeToSymbols3Dkernel<<<blockCount, blockSize>>>(dpQuant, dpData, 1.0f / quantizationStep, sizeX, sizeY, rowPitchSrc, slicePitchSrc, blockCountY);
	cudaCheckMsg("quantizeToSymbols3Dkernel execution failed");
}

void unquantizeFromSymbols3D(float* dpData, const ushort* dpQuant, float quantizationStep, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchDst, uint slicePitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY * sizeZ);

	unquantizeFromSymbols3Dkernel<<<blockCount, blockSize>>>(dpData, dpQuant, quantizationStep, sizeX, sizeY, rowPitchDst, slicePitchDst, blockCountY);
	cudaCheckMsg("unquantizeFromSymbols3Dkernel execution failed");
}



void quantizeDifferenceToSymbols2D(ushort* dpSymbols, const float* dpData, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	quantizeDifferenceToSymbols2Dkernel<<<blockCount, blockSize>>>(dpSymbols, dpData, 1.0f / quantizationStep, dpReference + channel, channelCount, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("quantizeDifferenceToSymbols2Dkernel execution failed");
}

void quantizeDifferenceToSymbolsRoundtrip2D(ushort* dpSymbols, float* dpData, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchSrc)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	quantizeDifferenceToSymbolsRoundtrip2Dkernel<<<blockCount, blockSize>>>(dpSymbols, dpData, quantizationStep, 1.0f / quantizationStep, dpReference + channel, channelCount, sizeX, sizeY, rowPitchSrc);
	cudaCheckMsg("quantizeDifferenceToSymbolsRoundtrip2Dkernel execution failed");
}

void unquantizeDifferenceFromSymbols2D(float* dpData, const ushort* dpSymbols, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchDst)
{
	dim3 blockSize(64, 4);
	uint blockCountX = (sizeX + blockSize.x - 1) / blockSize.x;
	uint blockCountY = (sizeY + blockSize.y - 1) / blockSize.y;
	dim3 blockCount(blockCountX, blockCountY);

	unquantizeDifferenceFromSymbols2Dkernel<<<blockCount, blockSize>>>(dpData, dpSymbols, quantizationStep, dpReference + channel, channelCount, sizeX, sizeY, rowPitchDst);
	cudaCheckMsg("unquantizeDifferenceFromSymbols2Dkernel execution failed");
}


}
