#ifndef __TUM3D_CUDACOMPRESS__QUANTIZE_CUH__
#define __TUM3D_CUDACOMPRESS__QUANTIZE_CUH__


#include "global.h"


__device__ inline uchar floatToByte(float val)
{
	return uchar(min(max(val + 128.5f, 0.0f), 255.0f));
}

__device__ inline float byteToFloat(uchar val)
{
	return float(val) - 128.0f;
}

// negative -> -1   non-negative -> 0
__device__ inline int getNegativeSign(short val)
{
	return (val >> 15);
}
//__device__ inline int getNegativeSign(int val)
//{
//	return (val >> 31);
//}

// negative -> -1   non-negative -> 1
__device__ inline float getFloatSign(short val)
{
	//if(val < 0) return -1.0f;
	//if(val > 0) return 1.0f;
	//return 0.0f;
	float t = val < 0 ? -1.0f : 0.0f;
	return val > 0 ? 1.0f : t;
}

__device__ inline ushort symbolize(short value)
{
	// map >= 0 to even, < 0 to odd
	return ushort(2 * abs(value) + getNegativeSign(value));
}

__device__ inline short unsymbolize(ushort symbol)
{
	short negative = symbol % short(2);
	// map even to >= 0, odd to < 0
	return (1 - 2 * negative) * ((symbol + negative) / 2);
}

__device__ inline short quantize(float value, float quantizationStepInv)
{
	// round-to-zero -> deadzone quantization with twice larger zero bin
	return short(value * quantizationStepInv);
}

__device__ inline float unquantize(short quant, float quantizationStep)
{
	// unquantize into middle of bin
	return (float(quant) + getFloatSign(quant) * 0.5f) * quantizationStep;
}

__device__ inline ushort quantizeToSymbol(float value, float quantizationStepInv)
{
	return symbolize(quantize(value, quantizationStepInv));
}

__device__ inline float unquantizeFromSymbol(ushort symbol, float quantizationStep)
{
	return unquantize(unsymbolize(symbol), quantizationStep);
}


#endif
