#ifndef __TUM3D_CUDACOMPRESS__QUANTIZE_H__
#define __TUM3D_CUDACOMPRESS__QUANTIZE_H__


#include "global.h"


namespace cudaCompress {

// quantize float to byte (byte array can be multi-channel; quant step 1; shift +128 to make unsigned)
void quantize2D(byte* dpQuant, uint channelCount, uint channel, const float* dpData, uint sizeX, uint sizeY, uint rowPitchSrc);
void unquantize2D(float* dpData, const byte* dpQuant, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchDst);


// convert signed shorts to symbols (>= 0 -> even, < 0 -> odd)
void symbolize2D(ushort* dpSymbols, const short* dpData, uint sizeX, uint sizeY, uint rowPitchSrc);
void unsymbolize2D(short* dpData, const ushort* dpSymbols, uint sizeX, uint sizeY, uint rowPitchDst);

void symbolize3D(ushort* dpSymbols, const short* dpData, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchSrc, uint slicePitchSrc);
void unsymbolize3D(short* dpData, const ushort* dpSymbols, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchDst, uint slicePitchDst);


// quantize float to symbols (>= 0 -> even, < 0 -> odd)
void quantizeToSymbols2D(ushort* dpSymbols, const float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchSrc);
void quantizeToSymbolsRoundtrip2D(ushort* dpSymbols, float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchSrc);
void unquantizeFromSymbols2D(float* dpData, const ushort* dpSymbols, float quantizationStep, uint sizeX, uint sizeY, uint rowPitchDst);

void quantizeToSymbols3D(ushort* dpSymbols, const float* dpData, float quantizationStep, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchSrc, uint slicePitchSrc);
void unquantizeFromSymbols3D(float* dpData, const ushort* dpSymbols, float quantizationStep, uint sizeX, uint sizeY, uint sizeZ, uint rowPitchDst, uint slicePitchDst);


// convert reference to float (with shift -128), subtract from data, quantize result to symbol
void quantizeDifferenceToSymbols2D(ushort* dpSymbols, const float* dpData, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchSrc);
void quantizeDifferenceToSymbolsRoundtrip2D(ushort* dpSymbols, float* dpData, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchSrc);
void unquantizeDifferenceFromSymbols2D(float* dpData, const ushort* dpSymbols, float quantizationStep, const byte* dpReference, uint channelCount, uint channel, uint sizeX, uint sizeY, uint rowPitchDst);

}


#endif
