#include "scan_plan.h"

#include <cstdlib>

#include <cuda_runtime_api.h>

#include "../cudaUtil.h"

#include "scan_globals.h"


ScanPlan::ScanPlan(size_t elemSizeBytes, size_t numElements)
: m_numElements(numElements),
  m_elemSizeBytes(elemSizeBytes),
  m_blockSums(0),
  m_numLevelsAllocated(0)
{
    size_t level = 0;
    size_t numElts = m_numElements;
	size_t blockSize = SCAN_ELTS_PER_THREAD * SCAN_CTA_SIZE;
    do
    {
        size_t numBlocks = (numElts + blockSize - 1) / blockSize;
        if (numBlocks > 1)
        {
            level++;
        }
        numElts = numBlocks;
    } while (numElts > 1);

    m_blockSums = (void**) malloc(level * sizeof(void*));

    m_numLevelsAllocated = level;
    numElts = m_numElements;

    level = 0;

    do
    {
        size_t numBlocks = (numElts + blockSize - 1) / blockSize;
        if (numBlocks > 1) 
        {
            cudaSafeCall(cudaMalloc((void**) &(m_blockSums[level++]), numBlocks * m_elemSizeBytes));
        }
        numElts = numBlocks;
    } while (numElts > 1);

    cudaCheckMsg("allocScanStorage");
}

ScanPlan::~ScanPlan()
{
    for (unsigned int i = 0; i < m_numLevelsAllocated; i++)
    {
        cudaFree(m_blockSums[i]);
    }

    cudaCheckMsg("freeScanStorage");

    free(m_blockSums);

    m_blockSums = 0;
    m_numLevelsAllocated = 0;
}
