#ifndef __pinned_allocator_h__
#define __pinned_allocator_h__


#include <cassert>
#include <limits>
#include <memory>

#include <cuda_runtime_api.h>


// implementation of an std::allocator that uses cudaMallocHost/cudaFreeHost
template<typename T>
class PinnedAllocator
{
public:
	typedef T value_type;
	typedef value_type* pointer;
	typedef value_type& reference;
	typedef const value_type* const_pointer;
	typedef const value_type& const_reference;

	typedef size_t size_type;
	typedef ptrdiff_t difference_type;

	PinnedAllocator() {}
	PinnedAllocator(const PinnedAllocator<T>& other) {}
	template<class _Other>
	PinnedAllocator(const PinnedAllocator<_Other>& other) {}

	template<class _Other>
	struct rebind
	{
		typedef PinnedAllocator<_Other> other;
	};

	pointer address(reference val) const
	{
		return (&val);
	}

	const_pointer address(const_reference val) const
	{
		return (&val);
	}

	pointer allocate(size_type n, const_pointer hint = 0)
	{
		if(n > max_size())
		{
			throw std::bad_alloc();
		}
		pointer result = 0;
		cudaError_t error = cudaMallocHost(reinterpret_cast<void**>&result, n * sizeof(T));
		if(cudaSuccess != error)
		{
			throw std::bad_alloc();
		}
		return result;
	}

	void deallocate(pointer p, size_type n)
	{
		cudaError_t error = cudaFreeHost(p);
		assert(cudaSuccess == error);
	}

	void construct(pointer p, const T& val)
	{	// construct object at p with value val
		void* _Vptr = p;
		::new (_Vptr) T(std::forward<T>(val));
	}

	void construct(pointer p, T&& val)
	{	// construct object at p with value val
		::new ((void*)p) T(std::forward<T>(val));
	}

	template<class Other>
	void construct(pointer p, Other&& val)
	{	// construct object at p with value val
		::new ((void*)p) T(std::forward<Other>(val));
	}

	void destroy(pointer p)
	{	// destroy object at p
		p->~T();
	}

	size_type max_size() const
	{
		return std::numeric_limits<size_type>::max() / sizeof(T);
	}
};


#endif
