#ifndef __entropy_h__
#define __entropy_h__


#include <cassert>
#include <cmath>
#include <limits>


template<typename T>
inline double computeEntropy(const T* pData, unsigned int count)
{
	assert(std::numeric_limits<T>::is_integer);
	assert(!std::numeric_limits<T>::is_signed);

	T max = 0;
	for(unsigned int i = 0; i < count; i++) {
		if(pData[i] > max) max = pData[i];
	}
	unsigned int symbolCount = max + 1;

	double *pProbabilities = new double[symbolCount];
	for(unsigned int i = 0; i < symbolCount; i++) {
		pProbabilities[i] = 0.0;
	}
	for(unsigned int i = 0; i < count; i++) {
		pProbabilities[pData[i]]++;
	}

	double sum = 0.0;
	for(unsigned int i = 0; i < symbolCount; i++) {
		pProbabilities[i] /= count;

		if (pProbabilities[i] > 0.0) {
			sum -= pProbabilities[i] * log(pProbabilities[i]);
		}
	}

	delete[] pProbabilities;

	return sum / log(2.0);
}


#endif
